/*
 * Title:        CloudSim Toolkit
 * Description:  CloudSim (Cloud Simulation) Toolkit for Modeling and Simulation of Clouds
 * Licence:      GPL - http://www.gnu.org/copyleft/gpl.html
 *
 * Copyright (c) 2009-2012, The University of Melbourne, Australia
 */

package org.cloudbus.cloudsim.CKP.datacenter;

import java.util.Hashtable;
import java.util.Map;


/**
 * This class represents an link (edge) from an graph
 * 
 * @author Thomas Hohnstein
 * @since CloudSim Toolkit 1.0
 */
public class DataCenterLink {

	/**
	 * id of the link src node-id
	 */
	private int srcNode ;

	/**
	 * id of the link dest node-id
	 */
	private int destNode ;

	/**
	 * representing the link-delay of the connection
	 */
	private float linkDelay = 0;

	private float linkBw;
	
	private boolean hasVisited = false;
	
	private  Hashtable<VMLink, Float> allocatedBw = new Hashtable<VMLink, Float>();
	
	public float backgroundBw = 0;
	
	public Map<Double, Double> hasAssigned = new Hashtable<Double, Double>();
	
	public double V = 1;
	
	public double temp_V_AVE;
	
	public double V_AVE;

	/**
	 * creates an new link-object
	 */
	public DataCenterLink(int srcNode, int destNode, float delay, float bw) {
		// lets initialize all internal attributes
		linkDelay = delay;
		this.srcNode = srcNode;
		this.destNode = destNode;
		linkBw = bw;
	}

	/**
	 * returns the node-ID from the SrcNode
	 * 
	 * @return nodeID
	 */
	public int getSrcNodeID() {
		return srcNode;
	}

	/**
	 * return the node-ID from the DestNode
	 * 
	 * @return nodeID
	 */
	public int getDestNodeID() {
		return destNode;
	}

	/**
	 * return the link-delay of the defined linke
	 * 
	 * @return the delay-amount
	 */
	public float getLinkDelay() {
		return linkDelay;
	}

	/**
	 * return the link-bw of the defined linke
	 * 
	 * @return the bw
	 */
	public float getLinkBw() {
		return linkBw;
	}

}
